@@ -290,58 +290,86 @@ func TestParse(t *testing.T) {
 
 func TestStaticDriverInfo(t *testing.T) {
 	for _, tc := range []struct {
-		Scenario string
-		input    string
-		needsMac bool
-		driver   string
-		boot     string
+		Scenario   string
+		input      string
+		needsMac   bool
+		driver     string
+		boot       string
+		management string
+		power      string
+		raid       string
+		vendor     string
 	}{
 		{
-			Scenario: "ipmi",
-			input:    "ipmi://192.168.122.1:6233",
-			needsMac: false,
-			driver:   "ipmi",
-			boot:     "ipxe",
+			Scenario:   "ipmi",
+			input:      "ipmi://192.168.122.1:6233",
+			needsMac:   false,
+			driver:     "ipmi",
+			boot:       "ipxe",
+			management: "",
+			power:      "",
+			raid:       "",
+			vendor:     "",
 		},
 
 		{
-			Scenario: "libvirt",
-			input:    "libvirt://192.168.122.1",
-			needsMac: true,
-			driver:   "ipmi",
-			boot:     "ipxe",
+			Scenario:   "libvirt",
+			input:      "libvirt://192.168.122.1",
+			needsMac:   true,
+			driver:     "ipmi",
+			boot:       "ipxe",
+			management: "",
+			power:      "",
+			raid:       "",
+			vendor:     "",
 		},
 
 		{
-			Scenario: "idrac",
-			input:    "idrac://192.168.122.1",
-			needsMac: false,
-			driver:   "idrac",
-			boot:     "ipxe",
+			Scenario:   "idrac",
+			input:      "idrac://192.168.122.1",
+			needsMac:   false,
+			driver:     "idrac",
+			boot:       "ipxe",
+			management: "",
+			power:      "",
+			raid:       "",
+			vendor:     "",
 		},
 
 		{
-			Scenario: "irmc",
-			input:    "irmc://192.168.122.1",
-			needsMac: false,
-			driver:   "irmc",
-			boot:     "pxe",
+			Scenario:   "irmc",
+			input:      "irmc://192.168.122.1",
+			needsMac:   false,
+			driver:     "irmc",
+			boot:       "pxe",
+			management: "",
+			power:      "",
+			raid:       "irmc",
+			vendor:     "",
 		},
 
 		{
-			Scenario: "redfish",
-			input:    "redfish://192.168.122.1",
-			needsMac: true,
-			driver:   "redfish",
-			boot:     "ipxe",
+			Scenario:   "redfish",
+			input:      "redfish://192.168.122.1",
+			needsMac:   true,
+			driver:     "redfish",
+			boot:       "ipxe",
+			management: "",
+			power:      "",
+			raid:       "",
+			vendor:     "",
 		},
 
 		{
-			Scenario: "redfish virtual media",
-			input:    "redfish-virtualmedia://192.168.122.1",
-			needsMac: true,
-			driver:   "redfish",
-			boot:     "redfish-virtual-media",
+			Scenario:   "redfish virtual media",
+			input:      "redfish-virtualmedia://192.168.122.1",
+			needsMac:   true,
+			driver:     "redfish",
+			boot:       "redfish-virtual-media",
+			management: "",
+			power:      "",
+			raid:       "",
+			vendor:     "",
 		},
 
 		{
@@ -353,15 +381,19 @@ func TestStaticDriverInfo(t *testing.T) {
 		},
 
 		{
-			Scenario: "idrac virtual media",
-			input:    "idrac-virtualmedia://192.168.122.1",
-			needsMac: true,
-			driver:   "idrac",
-			boot:     "idrac-redfish-virtual-media",
+			Scenario:   "idrac virtual media",
+			input:      "idrac-virtualmedia://192.168.122.1",
+			needsMac:   true,
+			driver:     "idrac",
+			boot:       "idrac-redfish-virtual-media",
+			management: "idrac-redfish",
+			power:      "idrac-redfish",
+			raid:       "no-raid",
+			vendor:     "no-vendor",
 		},
 	} {
 		t.Run(tc.Scenario, func(t *testing.T) {
-			acc, err := NewAccessDetails(tc.input)
+			acc, err := NewAccessDetails(tc.input, false)
 			if err != nil {
 				t.Fatalf("unexpected parse error: %v", err)
 			}
@@ -383,229 +415,249 @@ func TestDriverInfo(t *testing.T) {
 	for _, tc := range []struct {
 		Scenario string
 		input    string
-		expects  map[string]string
+		expects  map[string]interface{}
 	}{
 		{
 			Scenario: "ipmi default port",
 			input:    "ipmi://192.168.122.1",
-			expects: map[string]string{
-				"ipmi_port":     ipmiDefaultPort,
-				"ipmi_password": "",
-				"ipmi_username": "",
-				"ipmi_address":  "192.168.122.1",
+			expects: map[string]interface{}{
+				"ipmi_port":      ipmiDefaultPort,
+				"ipmi_password":  "",
+				"ipmi_username":  "",
+				"ipmi_address":   "192.168.122.1",
+				"ipmi_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac",
 			input:    "idrac://192.168.122.1",
-			expects: map[string]string{
-				"drac_address":  "192.168.122.1",
-				"drac_password": "",
-				"drac_username": "",
+			expects: map[string]interface{}{
+				"drac_address":   "192.168.122.1",
+				"drac_password":  "",
+				"drac_username":  "",
+				"drac_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac http",
 			input:    "idrac+http://192.168.122.1",
-			expects: map[string]string{
-				"drac_address":  "192.168.122.1",
-				"drac_protocol": "http",
-				"drac_password": "",
-				"drac_username": "",
+			expects: map[string]interface{}{
+				"drac_address":   "192.168.122.1",
+				"drac_protocol":  "http",
+				"drac_password":  "",
+				"drac_username":  "",
+				"drac_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac https",
 			input:    "idrac+https://192.168.122.1",
-			expects: map[string]string{
-				"drac_address":  "192.168.122.1",
-				"drac_protocol": "https",
-				"drac_password": "",
-				"drac_username": "",
+			expects: map[string]interface{}{
+				"drac_address":   "192.168.122.1",
+				"drac_protocol":  "https",
+				"drac_password":  "",
+				"drac_username":  "",
+				"drac_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac port and path http",
 			input:    "idrac://192.168.122.1:8080/foo",
-			expects: map[string]string{
-				"drac_address":  "192.168.122.1",
-				"drac_port":     "8080",
-				"drac_path":     "/foo",
-				"drac_password": "",
-				"drac_username": "",
+			expects: map[string]interface{}{
+				"drac_address":   "192.168.122.1",
+				"drac_port":      "8080",
+				"drac_path":      "/foo",
+				"drac_password":  "",
+				"drac_username":  "",
+				"drac_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac ipv6",
 			input:    "idrac://[fe80::fc33:62ff:fe83:8a76]/foo",
-			expects: map[string]string{
-				"drac_address":  "fe80::fc33:62ff:fe83:8a76",
-				"drac_path":     "/foo",
-				"drac_password": "",
-				"drac_username": "",
+			expects: map[string]interface{}{
+				"drac_address":   "fe80::fc33:62ff:fe83:8a76",
+				"drac_path":      "/foo",
+				"drac_password":  "",
+				"drac_username":  "",
+				"drac_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac ipv6 port and path",
 			input:    "idrac://[fe80::fc33:62ff:fe83:8a76]:8080/foo",
-			expects: map[string]string{
-				"drac_address":  "fe80::fc33:62ff:fe83:8a76",
-				"drac_port":     "8080",
-				"drac_path":     "/foo",
-				"drac_password": "",
-				"drac_username": "",
+			expects: map[string]interface{}{
+				"drac_address":   "fe80::fc33:62ff:fe83:8a76",
+				"drac_port":      "8080",
+				"drac_path":      "/foo",
+				"drac_password":  "",
+				"drac_username":  "",
+				"drac_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "irmc",
 			input:    "irmc://192.168.122.1",
-			expects: map[string]string{
-				"irmc_address":  "192.168.122.1",
-				"irmc_password": "",
-				"irmc_username": "",
+			expects: map[string]interface{}{
+				"irmc_address":   "192.168.122.1",
+				"irmc_password":  "",
+				"irmc_username":  "",
+				"irmc_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "irmc port",
 			input:    "irmc://192.168.122.1:8080",
-			expects: map[string]string{
-				"irmc_address":  "192.168.122.1",
-				"irmc_port":     "8080",
-				"irmc_password": "",
-				"irmc_username": "",
+			expects: map[string]interface{}{
+				"irmc_address":   "192.168.122.1",
+				"irmc_port":      "8080",
+				"irmc_password":  "",
+				"irmc_username":  "",
+				"irmc_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "irmc ipv6",
 			input:    "irmc://[fe80::fc33:62ff:fe83:8a76]",
-			expects: map[string]string{
-				"irmc_address":  "fe80::fc33:62ff:fe83:8a76",
-				"irmc_password": "",
-				"irmc_username": "",
+			expects: map[string]interface{}{
+				"irmc_address":   "fe80::fc33:62ff:fe83:8a76",
+				"irmc_password":  "",
+				"irmc_username":  "",
+				"irmc_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "irmc ipv6 port",
 			input:    "irmc://[fe80::fc33:62ff:fe83:8a76]:8080",
-			expects: map[string]string{
-				"irmc_address":  "fe80::fc33:62ff:fe83:8a76",
-				"irmc_port":     "8080",
-				"irmc_password": "",
-				"irmc_username": "",
+			expects: map[string]interface{}{
+				"irmc_address":   "fe80::fc33:62ff:fe83:8a76",
+				"irmc_port":      "8080",
+				"irmc_password":  "",
+				"irmc_username":  "",
+				"irmc_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish",
 			input:    "redfish://192.168.122.1/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://192.168.122.1",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish http",
 			input:    "redfish+http://192.168.122.1/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "http://192.168.122.1",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish https",
 			input:    "redfish+https://192.168.122.1/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://192.168.122.1",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish port",
 			input:    "redfish://192.168.122.1:8080/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://192.168.122.1:8080",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish ipv6",
 			input:    "redfish://[fe80::fc33:62ff:fe83:8a76]/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://[fe80::fc33:62ff:fe83:8a76]",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish ipv6 port",
 			input:    "redfish://[fe80::fc33:62ff:fe83:8a76]:8080/foo",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://[fe80::fc33:62ff:fe83:8a76]:8080",
 				"redfish_system_id": "/foo",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "Redfish virtual media",
 			input:    "redfish-virtualmedia://192.168.122.1/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://192.168.122.1",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "ilo5 virtual media",
 			input:    "ilo5-virtualmedia://192.168.122.1/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://192.168.122.1",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 
 		{
 			Scenario: "idrac virtual media",
 			input:    "idrac-virtualmedia://192.168.122.1/foo/bar",
-			expects: map[string]string{
+			expects: map[string]interface{}{
 				"redfish_address":   "https://192.168.122.1",
 				"redfish_system_id": "/foo/bar",
 				"redfish_password":  "",
 				"redfish_username":  "",
+				"redfish_verify_ca": false,
 			},
 		},
 	} {
 		t.Run(tc.Scenario, func(t *testing.T) {
-			acc, err := NewAccessDetails(tc.input)
+			acc, err := NewAccessDetails(tc.input, true)
 			if err != nil {
 				t.Fatalf("unexpected parse error: %v", err)
 			}
@@ -627,7 +679,7 @@ func TestDriverInfo(t *testing.T) {
 }
 
 func TestUnknownType(t *testing.T) {
-	acc, err := NewAccessDetails("foo://192.168.122.1")
+	acc, err := NewAccessDetails("foo://192.168.122.1", false)
 	if err == nil || acc != nil {
 		t.Fatalf("unexpected parse success")
 	}
