@@ -2,8 +2,10 @@ package baremetalhost
 
 import (
 	"context"
+	"encoding/json"
 	"flag"
 	"fmt"
+	"reflect"
 	"strings"
 	"time"
 
@@ -31,9 +33,9 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
+	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
-	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 )
 
@@ -119,12 +121,13 @@ type ReconcileBareMetalHost struct {
 // Instead of passing a zillion arguments to the action of a phase,
 // hold them in a context
 type reconcileInfo struct {
-	log            logr.Logger
-	host           *metal3v1alpha1.BareMetalHost
-	request        reconcile.Request
-	bmcCredsSecret *corev1.Secret
-	events         []corev1.Event
-	errorMessage   string
+	log               logr.Logger
+	host              *metal3v1alpha1.BareMetalHost
+	request           reconcile.Request
+	bmcCredsSecret    *corev1.Secret
+	events            []corev1.Event
+	errorMessage      string
+	postSaveCallbacks []func()
 }
 
 // match the provisioner.EventPublisher interface
@@ -132,19 +135,6 @@ func (info *reconcileInfo) publishEvent(reason, message string) {
 	info.events = append(info.events, info.host.NewEvent(reason, message))
 }
 
-// Action for one step of reconciliation.
-//
-// - Return a result if the host should be saved and requeued without error.
-// - Return error if there was an error.
-// - Return double nil if nothing was done and processing should continue.
-type reconcileAction func(info *reconcileInfo) (*reconcile.Result, error)
-
-// One step of reconciliation
-type reconcilePhase struct {
-	name   string
-	action reconcileAction
-}
-
 // Reconcile reads that state of the cluster for a BareMetalHost
 // object and makes changes based on the state read and what is in the
 // BareMetalHost.Spec TODO(user): Modify this Reconcile function to
@@ -154,6 +144,12 @@ type reconcilePhase struct {
 // is true, otherwise upon completion it will remove the work from the
 // queue.
 func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result reconcile.Result, err error) {
+	reconcileCounters.With(hostMetricLabels(request)).Inc()
+	defer func() {
+		if err != nil {
+			reconcileErrorCounter.Inc()
+		}
+	}()
 
 	reqLogger := log.WithValues("Request.Namespace",
 		request.Namespace, "Request.Name", request.Name)
@@ -174,6 +170,29 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 		return reconcile.Result{}, errors.Wrap(err, "could not load host data")
 	}
 
+	// Check if Status is empty and status annotation is present
+	// Manually restore data.
+	if !r.hostHasStatus(host) {
+		reqLogger.Info("Fetching Status from Annotation")
+		objStatus, err := r.getHostStatusFromAnnotation(host)
+		if err == nil && objStatus != nil {
+			host.Status = *objStatus
+			if host.Status.LastUpdated.IsZero() {
+				// Ensure the LastUpdated timestamp in set to avoid
+				// infinite loops if the annotation only contained
+				// part of the status information.
+				t := metav1.Now()
+				host.Status.LastUpdated = &t
+			}
+			errStatus := r.client.Status().Update(context.TODO(), host)
+			if errStatus != nil {
+				return reconcile.Result{}, errors.Wrap(err, "Could not restore status from annotation")
+			}
+			return reconcile.Result{Requeue: true}, nil
+		}
+		reqLogger.Info("No status cache found")
+	}
+
 	// NOTE(dhellmann): Handle a few steps outside of the phase
 	// structure because they require extra data lookup (like the
 	// credential checks) or have to be done "first" (like delete
@@ -195,109 +214,23 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 		return reconcile.Result{Requeue: true}, nil
 	}
 
-	// Handle delete operations
-	if !host.DeletionTimestamp.IsZero() {
-		result, err := r.deleteHost(request, host)
-		return result, err
-	}
-
 	// Retrieve the BMC details from the host spec and validate host
 	// BMC details and build the credentials for talking to the
 	// management controller.
 	bmcCreds, bmcCredsSecret, err := r.buildAndValidateBMCCredentials(request, host)
-	if err != nil {
-		switch err.(type) {
-		// We treat an empty bmc address and empty bmc credentials fields as a
-		// trigger the host needs to be put into a discovered status. We also set
-		// an error message (but not an error state) on the host so we can understand
-		// what we may be waiting on.  Editing the host to set these values will
-		// cause the host to be reconciled again so we do not Requeue.
-		case *EmptyBMCAddressError, *EmptyBMCSecretError:
-			dirty := host.SetOperationalStatus(metal3v1alpha1.OperationalStatusDiscovered)
-			if dirty {
-				// Set the host error message directly
-				// as we cannot use SetErrorCondition which
-				// overwrites our discovered state
-				host.Status.ErrorMessage = err.Error()
-				saveErr := r.saveStatus(host)
-				if saveErr != nil {
-					return reconcile.Result{Requeue: true}, saveErr
-				}
-				// Only publish the event if we do not have an error
-				// after saving so that we only publish one time.
-				r.publishEvent(request,
-					host.NewEvent("Discovered", fmt.Sprintf("Discovered host with unusable BMC details: %s", err.Error())))
-			}
-			return reconcile.Result{}, nil
-		// In the event a credential secret is defined, but we cannot find it
-		// we requeue the host as we will not know if they create the secret
-		// at some point in the future.
-		case *ResolveBMCSecretRefError:
-			changed, saveErr := r.setErrorCondition(request, host, err.Error())
-			if saveErr != nil {
-				return reconcile.Result{Requeue: true}, saveErr
-			}
-			if changed {
-				// Only publish the event if we do not have an error
-				// after saving so that we only publish one time.
-				r.publishEvent(request, host.NewEvent("BMCCredentialError", err.Error()))
-			}
-			return reconcile.Result{Requeue: true, RequeueAfter: hostErrorRetryDelay}, nil
-		// If we have found the secret but it is missing the required fields
-		// or the BMC address is defined but malformed we set the
-		// host into an error state but we do not Requeue it
-		// as fixing the secret or the host BMC info will trigger
-		// the host to be reconciled again
-		case *bmc.CredentialsValidationError, *bmc.UnknownBMCTypeError:
-			_, saveErr := r.setErrorCondition(request, host, err.Error())
-			if saveErr != nil {
-				return reconcile.Result{Requeue: true}, saveErr
-			}
-			// Only publish the event if we do not have an error
-			// after saving so that we only publish one time.
-			r.publishEvent(request, host.NewEvent("BMCCredentialError", err.Error()))
-			return reconcile.Result{}, nil
-		default:
-			return reconcile.Result{}, errors.Wrap(err, "An unhandled failure occurred with the BMC secret")
+	if err != nil || bmcCreds == nil {
+		if !host.DeletionTimestamp.IsZero() {
+			// If we are in the process of deletion, try with empty credentials
+			bmcCreds = &bmc.Credentials{}
+			bmcCredsSecret = &corev1.Secret{}
+		} else {
+			return r.credentialsErrorResult(err, request, host)
 		}
 	}
 
-	// Pick the action to perform
-	var actionName metal3v1alpha1.ProvisioningState
-	switch {
-	case !host.Status.GoodCredentials.Match(*bmcCredsSecret):
-		actionName = metal3v1alpha1.StateRegistering
-	case host.Spec.ExternallyProvisioned:
-		actionName = metal3v1alpha1.StateExternallyProvisioned
-	case host.NeedsHardwareInspection():
-		actionName = metal3v1alpha1.StateInspecting
-	case host.NeedsHardwareProfile():
-		actionName = metal3v1alpha1.StateMatchProfile
-	case host.NeedsProvisioning():
-		actionName = metal3v1alpha1.StateProvisioning
-	case host.NeedsDeprovisioning():
-		actionName = metal3v1alpha1.StateDeprovisioning
-	case host.WasProvisioned():
-		actionName = metal3v1alpha1.StateProvisioned
-	default:
-		actionName = metal3v1alpha1.StateReady
-	}
-
-	if actionName != host.Status.Provisioning.State {
-		reqLogger.Info("changing provisioning state",
-			"old", host.Status.Provisioning.State,
-			"new", actionName,
-		)
-		host.Status.Provisioning.State = actionName
-		if err := r.saveStatus(host); err != nil {
-			return reconcile.Result{}, errors.Wrap(err,
-				fmt.Sprintf("failed to save host status after handling %q", actionName))
-		}
-		return reconcile.Result{Requeue: true}, nil
-	}
-
+	initialState := host.Status.Provisioning.State
 	info := &reconcileInfo{
-		log:            reqLogger.WithValues("provisioningState", actionName),
+		log:            reqLogger.WithValues("provisioningState", initialState),
 		host:           host,
 		request:        request,
 		bmcCredsSecret: bmcCredsSecret,
@@ -307,43 +240,32 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 		return reconcile.Result{}, errors.Wrap(err, "failed to create provisioner")
 	}
 
-	switch actionName {
-	case metal3v1alpha1.StateRegistering:
-		result, err = r.actionRegistering(prov, info)
-	case metal3v1alpha1.StateInspecting:
-		result, err = r.actionInspecting(prov, info)
-	case metal3v1alpha1.StateMatchProfile:
-		result, err = r.actionMatchProfile(prov, info)
-	case metal3v1alpha1.StateProvisioning:
-		result, err = r.actionProvisioning(prov, info)
-	case metal3v1alpha1.StateDeprovisioning:
-		result, err = r.actionDeprovisioning(prov, info)
-	case metal3v1alpha1.StateProvisioned:
-		result, err = r.actionManageSteadyState(prov, info)
-	case metal3v1alpha1.StateReady:
-		result, err = r.actionManageReady(prov, info)
-	case metal3v1alpha1.StateExternallyProvisioned:
-		result, err = r.actionManageSteadyState(prov, info)
-	default:
-		// Probably a provisioning error state?
-		return reconcile.Result{}, fmt.Errorf("Unrecognized action %q", actionName)
-	}
+	stateMachine := newHostStateMachine(host, r, prov)
+	actResult := stateMachine.ReconcileState(info)
+	result, err = actResult.Result()
 
 	if err != nil {
-		return reconcile.Result{}, errors.Wrap(err, fmt.Sprintf("action %q failed", actionName))
+		err = errors.Wrap(err, fmt.Sprintf("action %q failed", initialState))
+		return
 	}
 
-	// Only save status when we're told to requeue, otherwise we
+	// Only save status when we're told to, otherwise we
 	// introduce an infinite loop reconciling the same object over and
 	// over when there is an unrecoverable error (tracked through the
 	// error state of the host).
-	if result.Requeue {
+	if actResult.Dirty() {
+
+		// Save Host
 		info.log.Info("saving host status",
 			"operational status", host.OperationalStatus(),
 			"provisioning state", host.Status.Provisioning.State)
-		if err = r.saveStatus(host); err != nil {
+		if err = r.saveHostStatus(host); err != nil {
 			return reconcile.Result{}, errors.Wrap(err,
-				fmt.Sprintf("failed to save host status after %q", actionName))
+				fmt.Sprintf("failed to save host status after %q", initialState))
+		}
+
+		for _, cb := range info.postSaveCallbacks {
+			cb()
 		}
 	}
 
@@ -351,106 +273,142 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 		r.publishEvent(request, e)
 	}
 
-	if host.HasError() {
-		// We have tried to do something that failed in a way we
-		// assume is not retryable, so do not proceed to any other
-		// steps.
-		info.log.Info("stopping on host error", "message", host.Status.ErrorMessage)
-		return reconcile.Result{}, nil
-	}
-
-	info.log.Info("done",
-		"requeue", result.Requeue,
-		"after", result.RequeueAfter,
-	)
-	return result, nil
+	logResult(info, result)
+	return
 }
 
-// Handle all delete cases
-func (r *ReconcileBareMetalHost) deleteHost(request reconcile.Request, host *metal3v1alpha1.BareMetalHost) (result reconcile.Result, err error) {
+func logResult(info *reconcileInfo, result reconcile.Result) {
+	if result.Requeue || result.RequeueAfter != 0 ||
+		!utils.StringInList(info.host.Finalizers,
+			metal3v1alpha1.BareMetalHostFinalizer) {
+		info.log.Info("done",
+			"requeue", result.Requeue,
+			"after", result.RequeueAfter)
+	} else {
+		info.log.Info("stopping on host error",
+			"message", info.host.Status.ErrorMessage)
+	}
+}
 
-	reqLogger := log.WithValues("Request.Namespace",
-		request.Namespace, "Request.Name", request.Name)
+func recordActionFailure(info *reconcileInfo, errorType metal3v1alpha1.ErrorType, errorMessage string) actionFailed {
+	dirty := info.host.SetErrorMessage(errorType, errorMessage)
+	if dirty {
+		eventType := map[metal3v1alpha1.ErrorType]string{
+			metal3v1alpha1.RegistrationError:    "RegistrationError",
+			metal3v1alpha1.InspectionError:      "InspectionError",
+			metal3v1alpha1.ProvisioningError:    "ProvisioningError",
+			metal3v1alpha1.PowerManagementError: "PowerManagementError",
+		}[errorType]
 
-	reqLogger.Info(
+		counter := actionFailureCounters.WithLabelValues(eventType)
+		info.postSaveCallbacks = append(info.postSaveCallbacks, counter.Inc)
+
+		info.publishEvent(eventType, errorMessage)
+	}
+	return actionFailed{dirty: dirty, ErrorType: errorType}
+}
+
+func (r *ReconcileBareMetalHost) credentialsErrorResult(err error, request reconcile.Request, host *metal3v1alpha1.BareMetalHost) (reconcile.Result, error) {
+	switch err.(type) {
+	// We treat an empty bmc address and empty bmc credentials fields as a
+	// trigger the host needs to be put into a discovered status. We also set
+	// an error message (but not an error state) on the host so we can understand
+	// what we may be waiting on.  Editing the host to set these values will
+	// cause the host to be reconciled again so we do not Requeue.
+	case *EmptyBMCAddressError, *EmptyBMCSecretError:
+		credentialsInvalid.Inc()
+		dirty := host.SetOperationalStatus(metal3v1alpha1.OperationalStatusDiscovered)
+		if dirty {
+			// Set the host error message directly
+			// as we cannot use SetErrorCondition which
+			// overwrites our discovered state
+			host.Status.ErrorMessage = err.Error()
+			host.Status.ErrorType = ""
+			saveErr := r.saveHostStatus(host)
+			if saveErr != nil {
+				return reconcile.Result{Requeue: true}, saveErr
+			}
+			// Only publish the event if we do not have an error
+			// after saving so that we only publish one time.
+			r.publishEvent(request,
+				host.NewEvent("Discovered", fmt.Sprintf("Discovered host with unusable BMC details: %s", err.Error())))
+		}
+		return reconcile.Result{}, nil
+	// In the event a credential secret is defined, but we cannot find it
+	// we requeue the host as we will not know if they create the secret
+	// at some point in the future.
+	case *ResolveBMCSecretRefError:
+		credentialsMissing.Inc()
+		changed, saveErr := r.setErrorCondition(request, host, metal3v1alpha1.RegistrationError, err.Error())
+		if saveErr != nil {
+			return reconcile.Result{Requeue: true}, saveErr
+		}
+		if changed {
+			// Only publish the event if we do not have an error
+			// after saving so that we only publish one time.
+			r.publishEvent(request, host.NewEvent("BMCCredentialError", err.Error()))
+		}
+		return reconcile.Result{Requeue: true, RequeueAfter: hostErrorRetryDelay}, nil
+	// If we have found the secret but it is missing the required fields
+	// or the BMC address is defined but malformed we set the
+	// host into an error state but we do not Requeue it
+	// as fixing the secret or the host BMC info will trigger
+	// the host to be reconciled again
+	case *bmc.CredentialsValidationError, *bmc.UnknownBMCTypeError:
+		credentialsInvalid.Inc()
+		_, saveErr := r.setErrorCondition(request, host, metal3v1alpha1.RegistrationError, err.Error())
+		if saveErr != nil {
+			return reconcile.Result{Requeue: true}, saveErr
+		}
+		// Only publish the event if we do not have an error
+		// after saving so that we only publish one time.
+		r.publishEvent(request, host.NewEvent("BMCCredentialError", err.Error()))
+		return reconcile.Result{}, nil
+	default:
+		unhandledCredentialsError.Inc()
+		return reconcile.Result{}, errors.Wrap(err, "An unhandled failure occurred with the BMC secret")
+	}
+}
+
+// Manage deletion of the host
+func (r *ReconcileBareMetalHost) actionDeleting(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
+	info.log.Info(
 		"marked to be deleted",
-		"timestamp", host.DeletionTimestamp,
+		"timestamp", info.host.DeletionTimestamp,
 	)
 
 	// no-op if finalizer has been removed.
-	if !utils.StringInList(host.Finalizers, metal3v1alpha1.BareMetalHostFinalizer) {
-		reqLogger.Info("ready to be deleted")
-		// There is nothing to save and no reason to requeue since we
-		// are being deleted.
-		return reconcile.Result{}, nil
-	}
-
-	// Retrieve the BMC secret from Kubernetes for this host and
-	// try and build credentials.  If we fail, resort to an empty
-	// credentials object to give the provisioner
-	bmcCreds, _, err := r.buildAndValidateBMCCredentials(request, host)
-	if err != nil || bmcCreds == nil {
-		bmcCreds = &bmc.Credentials{}
-	}
-
-	eventPublisher := func(reason, message string) {
-		r.publishEvent(request, host.NewEvent(reason, message))
-	}
-
-	prov, err := r.provisionerFactory(host, *bmcCreds, eventPublisher)
-	if err != nil {
-		return result, errors.Wrap(err, "failed to create provisioner")
-	}
-
-	if host.NeedsDeprovisioning() {
-		reqLogger.Info("deprovisioning before deleting")
-		provResult, err := prov.Deprovision()
-		if err != nil {
-			return result, errors.Wrap(err, "failed to deprovision")
-		}
-		if provResult.Dirty {
-			err = r.saveStatus(host)
-			if err != nil {
-				return result, errors.Wrap(err, "failed to save host after deprovisioning")
-			}
-			result.Requeue = true
-			result.RequeueAfter = provResult.RequeueAfter
-			return result, nil
-		}
-	} else {
-		reqLogger.Info("no need to deprovision before deleting")
+	if !utils.StringInList(info.host.Finalizers, metal3v1alpha1.BareMetalHostFinalizer) {
+		info.log.Info("ready to be deleted")
+		return deleteComplete{}
 	}
 
 	provResult, err := prov.Delete()
 	if err != nil {
-		return result, errors.Wrap(err, "failed to delete")
+		return actionError{errors.Wrap(err, "failed to delete")}
 	}
 	if provResult.Dirty {
-		err = r.saveStatus(host)
+		err = r.saveHostStatus(info.host)
 		if err != nil {
-			return result, errors.Wrap(err, "failed to save host after deleting")
+			return actionError{errors.Wrap(err, "failed to save host after deleting")}
 		}
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	// Remove finalizer to allow deletion
-	host.Finalizers = utils.FilterStringFromList(
-		host.Finalizers, metal3v1alpha1.BareMetalHostFinalizer)
-	reqLogger.Info("cleanup is complete, removed finalizer",
-		"remaining", host.Finalizers)
-	if err := r.client.Update(context.Background(), host); err != nil {
-		return result, errors.Wrap(err, "failed to remove finalizer")
+	info.host.Finalizers = utils.FilterStringFromList(
+		info.host.Finalizers, metal3v1alpha1.BareMetalHostFinalizer)
+	info.log.Info("cleanup is complete, removed finalizer",
+		"remaining", info.host.Finalizers)
+	if err := r.client.Update(context.Background(), info.host); err != nil {
+		return actionError{errors.Wrap(err, "failed to remove finalizer")}
 	}
 
-	return result, nil
+	return deleteComplete{}
 }
 
 // Test the credentials by connecting to the management controller.
-func (r *ReconcileBareMetalHost) actionRegistering(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
-	var provResult provisioner.Result
-
+func (r *ReconcileBareMetalHost) actionRegistering(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	info.log.Info("registering and validating access to management controller",
 		"credentials", info.host.Status.TriedCredentials)
 
@@ -458,32 +416,25 @@ func (r *ReconcileBareMetalHost) actionRegistering(prov provisioner.Provisioner,
 	if credsChanged {
 		info.log.Info("new credentials")
 		info.host.UpdateTriedCredentials(*info.bmcCredsSecret)
+		info.postSaveCallbacks = append(info.postSaveCallbacks, updatedCredentials.Inc)
 	}
 
-	provResult, err = prov.ValidateManagementAccess(credsChanged)
+	provResult, err := prov.ValidateManagementAccess(credsChanged)
 	if err != nil {
-		return result, errors.Wrap(err, "failed to validate BMC access")
+		noManagementAccess.Inc()
+		return actionError{errors.Wrap(err, "failed to validate BMC access")}
 	}
 
 	info.log.Info("response from validate", "provResult", provResult)
 
 	if provResult.ErrorMessage != "" {
-		info.host.Status.Provisioning.State = metal3v1alpha1.StateRegistrationError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("RegistrationError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.RegistrationError, provResult.ErrorMessage)
 	}
 
 	if provResult.Dirty {
-		// Set Requeue true as well as RequeueAfter in case the delay
-		// is 0.
 		info.log.Info("host not ready", "wait", provResult.RequeueAfter)
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	// Reaching this point means the credentials are valid and worked,
@@ -501,46 +452,36 @@ func (r *ReconcileBareMetalHost) actionRegistering(prov provisioner.Provisioner,
 			"Registered host that was externally provisioned")
 	}
 
-	result.Requeue = true
-	return result, nil
+	return actionComplete{}
 }
 
 // Ensure we have the information about the hardware on the host.
-func (r *ReconcileBareMetalHost) actionInspecting(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
-	var provResult provisioner.Result
-	var details *metal3v1alpha1.HardwareDetails
-
+func (r *ReconcileBareMetalHost) actionInspecting(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	info.log.Info("inspecting hardware")
 
-	provResult, details, err = prov.InspectHardware()
+	provResult, details, err := prov.InspectHardware()
 	if err != nil {
-		return result, errors.Wrap(err, "hardware inspection failed")
+		return actionError{errors.Wrap(err, "hardware inspection failed")}
 	}
 
 	if provResult.ErrorMessage != "" {
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("InspectionError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.InspectionError, provResult.ErrorMessage)
 	}
 
 	if details != nil {
 		info.host.Status.HardwareDetails = details
-		result.Requeue = true
-		return result, nil
+		return actionComplete{}
 	}
 
 	if provResult.Dirty {
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
+		return actionContinue{provResult.RequeueAfter}
 	}
 
-	return result, nil
+	return actionFailed{}
 }
 
-func (r *ReconcileBareMetalHost) actionMatchProfile(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
+func (r *ReconcileBareMetalHost) actionMatchProfile(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 
 	var hardwareProfile string
 
@@ -551,10 +492,10 @@ func (r *ReconcileBareMetalHost) actionMatchProfile(prov provisioner.Provisioner
 		info.log.Info("using spec value for profile name",
 			"name", info.host.Spec.HardwareProfile)
 		hardwareProfile = info.host.Spec.HardwareProfile
-		_, err = hardware.GetProfile(hardwareProfile)
+		_, err := hardware.GetProfile(hardwareProfile)
 		if err != nil {
 			info.log.Info("invalid hardware profile", "profile", hardwareProfile)
-			return result, err
+			return actionError{err}
 		}
 	}
 
@@ -578,21 +519,13 @@ func (r *ReconcileBareMetalHost) actionMatchProfile(prov provisioner.Provisioner
 	if info.host.SetHardwareProfile(hardwareProfile) {
 		info.log.Info("updating hardware profile", "profile", hardwareProfile)
 		info.publishEvent("ProfileSet", fmt.Sprintf("Hardware profile set: %s", hardwareProfile))
-		info.host.ClearError()
-		result.Requeue = true
-		return result, nil
 	}
-
-	// Line up a requeue if we could provision
-	result.Requeue = info.host.NeedsProvisioning()
-
-	return result, nil
+	info.host.ClearError()
+	return actionComplete{}
 }
 
 // Start/continue provisioning if we need to.
-func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
-	var provResult provisioner.Result
-
+func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	getUserData := func() (string, error) {
 		if info.host.Spec.UserData == nil {
 			info.log.Info("no user data for host")
@@ -604,7 +537,7 @@ func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner
 			Name:      info.host.Spec.UserData.Name,
 			Namespace: info.host.Spec.UserData.Namespace,
 		}
-		err = r.client.Get(context.TODO(), key, userDataSecret)
+		err := r.client.Get(context.TODO(), key, userDataSecret)
 		if err != nil {
 			return "", errors.Wrap(err,
 				"failed to fetch user data from secret reference")
@@ -614,19 +547,14 @@ func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner
 
 	info.log.Info("provisioning")
 
-	provResult, err = prov.Provision(getUserData)
+	provResult, err := prov.Provision(getUserData)
 	if err != nil {
-		return result, errors.Wrap(err, "failed to provision")
+		return actionError{errors.Wrap(err, "failed to provision")}
 	}
 
 	if provResult.ErrorMessage != "" {
 		info.log.Info("handling provisioning error in controller")
-		info.host.Status.Provisioning.State = metal3v1alpha1.StateProvisioningError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("ProvisioningError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.ProvisioningError, provResult.ErrorMessage)
 	}
 
 	if provResult.Dirty {
@@ -634,9 +562,7 @@ func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner
 		// to return false, indicating that it has no more work to
 		// do.
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	// If the provisioner had no work, ensure the image settings match.
@@ -647,79 +573,58 @@ func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner
 
 	// After provisioning we always requeue to ensure we enter the
 	// "provisioned" state and start monitoring power status.
-	result.Requeue = true
-
-	return result, nil
+	return actionComplete{}
 }
 
-func (r *ReconcileBareMetalHost) actionDeprovisioning(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
-	var provResult provisioner.Result
-
+func (r *ReconcileBareMetalHost) actionDeprovisioning(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	info.log.Info("deprovisioning")
 
-	if provResult, err = prov.Deprovision(); err != nil {
-		return result, errors.Wrap(err, "failed to deprovision")
+	provResult, err := prov.Deprovision()
+	if err != nil {
+		return actionError{errors.Wrap(err, "failed to deprovision")}
 	}
 
 	if provResult.ErrorMessage != "" {
-		info.host.Status.Provisioning.State = metal3v1alpha1.StateProvisioningError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("ProvisioningError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.ProvisioningError, provResult.ErrorMessage)
 	}
 
 	if provResult.Dirty {
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	// After the provisioner is done, clear the image settings so we
 	// transition to the next state.
 	info.host.Status.Provisioning.Image = metal3v1alpha1.Image{}
 
-	// After deprovisioning we always requeue to ensure we enter the
-	// "ready" state and start monitoring power status.
-	result.Requeue = true
-
-	return result, nil
+	return actionComplete{}
 }
 
 // Check the current power status against the desired power status.
-func (r *ReconcileBareMetalHost) manageHostPower(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
+func (r *ReconcileBareMetalHost) manageHostPower(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	var provResult provisioner.Result
 
 	// Check the current status and save it before trying to update it.
-	if provResult, err = prov.UpdateHardwareState(); err != nil {
-		return result, errors.Wrap(err, "failed to update the host power status")
+	provResult, err := prov.UpdateHardwareState()
+	if err != nil {
+		return actionError{errors.Wrap(err, "failed to update the host power status")}
 	}
 
 	if provResult.ErrorMessage != "" {
-		info.host.Status.Provisioning.State = metal3v1alpha1.StatePowerManagementError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("PowerManagementError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.PowerManagementError, provResult.ErrorMessage)
 	}
 
 	if provResult.Dirty {
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	// Power state needs to be monitored regularly, so if we leave
 	// this function without an error we always want to requeue after
 	// a delay.
-	result.RequeueAfter = time.Second * 60
-
+	steadyStateResult := actionContinue{time.Second * 60}
 	if info.host.Status.PoweredOn == info.host.Spec.Online {
-		return result, nil
+		return steadyStateResult
 	}
 
 	info.log.Info("power state change needed",
@@ -732,33 +637,32 @@ func (r *ReconcileBareMetalHost) manageHostPower(prov provisioner.Provisioner, i
 		provResult, err = prov.PowerOff()
 	}
 	if err != nil {
-		return result, errors.Wrap(err, "failed to manage power state of host")
+		return actionError{errors.Wrap(err, "failed to manage power state of host")}
 	}
 
 	if provResult.ErrorMessage != "" {
-		info.host.Status.Provisioning.State = metal3v1alpha1.StatePowerManagementError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("PowerManagementError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.PowerManagementError, provResult.ErrorMessage)
 	}
 
 	if provResult.Dirty {
+		info.postSaveCallbacks = append(info.postSaveCallbacks, func() {
+			metricLabels := hostMetricLabels(info.request)
+			if info.host.Spec.Online {
+				metricLabels[labelPowerOnOff] = "on"
+			} else {
+				metricLabels[labelPowerOnOff] = "off"
+			}
+			powerChangeAttempts.With(metricLabels).Inc()
+		})
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	// The provisioner did not have to do anything to change the power
 	// state and there were no errors, so reflect the new state in the
 	// host status field.
 	info.host.Status.PoweredOn = info.host.Spec.Online
-	result.Requeue = true
-
-	return result, nil
-
+	return steadyStateResult
 }
 
 // A host reaching this action handler should be provisioned or
@@ -766,25 +670,18 @@ func (r *ReconcileBareMetalHost) manageHostPower(prov provisioner.Provisioner, i
 // user takes further action. Both of those states mean that it has
 // been registered with the provisioner once, so we use the Adopt()
 // API to ensure that is still true. Then we monitor its power status.
-func (r *ReconcileBareMetalHost) actionManageSteadyState(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
+func (r *ReconcileBareMetalHost) actionManageSteadyState(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 
 	provResult, err := prov.Adopt()
 	if err != nil {
-		return
+		return actionError{err}
 	}
 	if provResult.ErrorMessage != "" {
-		info.host.Status.Provisioning.State = metal3v1alpha1.StateRegistrationError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("RegistrationError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.RegistrationError, provResult.ErrorMessage)
 	}
 	if provResult.Dirty {
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
 	return r.manageHostPower(prov, info)
@@ -796,50 +693,123 @@ func (r *ReconcileBareMetalHost) actionManageSteadyState(prov provisioner.Provis
 // ValidateManagementAccess() to ensure that is still true. We don't
 // use Adopt() because we don't want Ironic to treat the host as
 // having been provisioned. Then we monitor its power status.
-func (r *ReconcileBareMetalHost) actionManageReady(prov provisioner.Provisioner, info *reconcileInfo) (result reconcile.Result, err error) {
+func (r *ReconcileBareMetalHost) actionManageReady(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 
 	// We always pass false for credentialsChanged because if they had
 	// changed we would have ended up in actionRegister() instead of
 	// here.
 	provResult, err := prov.ValidateManagementAccess(false)
 	if err != nil {
-		return
+		return actionError{err}
 	}
 	if provResult.ErrorMessage != "" {
-		info.host.Status.Provisioning.State = metal3v1alpha1.StateRegistrationError
-		if info.host.SetErrorMessage(provResult.ErrorMessage) {
-			info.publishEvent("RegistrationError", provResult.ErrorMessage)
-			result.Requeue = true
-		}
-		return result, nil
+		return recordActionFailure(info, metal3v1alpha1.RegistrationError, provResult.ErrorMessage)
 	}
 	if provResult.Dirty {
 		info.host.ClearError()
-		result.Requeue = true
-		result.RequeueAfter = provResult.RequeueAfter
-		return result, nil
+		return actionContinue{provResult.RequeueAfter}
 	}
 
+	if info.host.NeedsProvisioning() {
+		info.host.ClearError()
+		return actionComplete{}
+	}
 	return r.manageHostPower(prov, info)
 }
 
-func (r *ReconcileBareMetalHost) saveStatus(host *metal3v1alpha1.BareMetalHost) error {
+func (r *ReconcileBareMetalHost) saveHostStatus(host *metal3v1alpha1.BareMetalHost) error {
 	t := metav1.Now()
 	host.Status.LastUpdated = &t
-	return r.client.Status().Update(context.TODO(), host)
+
+	if err := r.saveHostAnnotation(host); err != nil {
+		return err
+	}
+
+	//Refetch host again
+	obj := host.Status.DeepCopy()
+	err := r.client.Get(context.TODO(),
+		client.ObjectKey{
+			Name:      host.Name,
+			Namespace: host.Namespace,
+		},
+		host,
+	)
+	if err != nil {
+		return errors.Wrap(err, "Failed to update Status annotation")
+	}
+	host.Status = *obj
+	err = r.client.Status().Update(context.TODO(), host)
+	return err
 }
 
-func (r *ReconcileBareMetalHost) setErrorCondition(request reconcile.Request, host *metal3v1alpha1.BareMetalHost, message string) (changed bool, err error) {
+func (r *ReconcileBareMetalHost) saveHostAnnotation(host *metal3v1alpha1.BareMetalHost) error {
+	//Repopulate annotation again
+	objStatus, err := r.getHostStatusFromAnnotation(host)
+	if err != nil {
+		return err
+	}
+
+	if objStatus != nil {
+		// This value is copied to avoid continually updating the annotation
+		objStatus.LastUpdated = host.Status.LastUpdated
+		if reflect.DeepEqual(host.Status, *objStatus) {
+			return nil
+		}
+	}
+
+	delete(host.Annotations, metal3v1alpha1.StatusAnnotation)
+	newAnnotation, err := marshalStatusAnnotation(&host.Status)
+	if err != nil {
+		return err
+	}
+	if host.Annotations == nil {
+		host.Annotations = make(map[string]string)
+	}
+	host.Annotations[metal3v1alpha1.StatusAnnotation] = string(newAnnotation)
+	return r.client.Update(context.TODO(), host.DeepCopy())
+}
+
+func marshalStatusAnnotation(status *metal3v1alpha1.BareMetalHostStatus) ([]byte, error) {
+	newAnnotation, err := json.Marshal(status)
+	if err != nil {
+		return []byte{}, errors.Wrap(err, "failed to marshall status annotation")
+	}
+	return newAnnotation, nil
+}
+
+func unmarshalStatusAnnotation(content []byte) (*metal3v1alpha1.BareMetalHostStatus, error) {
+	objStatus := &metal3v1alpha1.BareMetalHostStatus{}
+	if err := json.Unmarshal(content, objStatus); err != nil {
+		return nil, errors.Wrap(err, "Failed to fetch Status from annotation")
+	}
+	return objStatus, nil
+}
+
+// extract host from Status annotation
+func (r *ReconcileBareMetalHost) getHostStatusFromAnnotation(host *metal3v1alpha1.BareMetalHost) (*metal3v1alpha1.BareMetalHostStatus, error) {
+	annotations := host.GetAnnotations()
+	content := []byte(annotations[metal3v1alpha1.StatusAnnotation])
+	if annotations[metal3v1alpha1.StatusAnnotation] == "" {
+		return nil, nil
+	}
+	objStatus, err := unmarshalStatusAnnotation(content)
+	if err != nil {
+		return nil, err
+	}
+	return objStatus, nil
+}
+
+func (r *ReconcileBareMetalHost) setErrorCondition(request reconcile.Request, host *metal3v1alpha1.BareMetalHost, errType metal3v1alpha1.ErrorType, message string) (changed bool, err error) {
 	reqLogger := log.WithValues("Request.Namespace",
 		request.Namespace, "Request.Name", request.Name)
 
-	changed = host.SetErrorMessage(message)
+	changed = host.SetErrorMessage(errType, message)
 	if changed {
 		reqLogger.Info(
 			"adding error message",
 			"message", message,
 		)
-		err = r.saveStatus(host)
+		err = r.saveHostStatus(host)
 		if err != nil {
 			err = errors.Wrap(err, "failed to update error message")
 		}
@@ -897,7 +867,7 @@ func (r *ReconcileBareMetalHost) buildAndValidateBMCCredentials(request reconcil
 	// more in-depth checking on the url to ensure it is
 	// a valid bmc address, returning a bmc.UnknownBMCTypeError
 	// if it is not conformant
-	_, err = bmc.NewAccessDetails(host.Spec.BMC.Address)
+	_, err = bmc.NewAccessDetails(host.Spec.BMC.Address, host.Spec.BMC.DisableCertificateVerification)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -946,6 +916,10 @@ func (r *ReconcileBareMetalHost) publishEvent(request reconcile.Request, event c
 	return
 }
 
+func (r *ReconcileBareMetalHost) hostHasStatus(host *metal3v1alpha1.BareMetalHost) bool {
+	return !host.Status.LastUpdated.IsZero()
+}
+
 func hostHasFinalizer(host *metal3v1alpha1.BareMetalHost) bool {
 	return utils.StringInList(host.Finalizers, metal3v1alpha1.BareMetalHostFinalizer)
 }
